"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const classes_1 = require("../../classes/index.js");
const state_1 = require("../../state/index.js");
const custom_oracle_1 = require("../../utils/custom-oracle/index.js");
const web3_js_1 = require("@solana/web3.js");
const bn_js_1 = require("bn.js");
const constants_1 = require("../constants.js");
const prices_1 = require("./prices.js");
jest.mock("@/utils/custom-oracle");
jest.mock("../constants", () => {
    const originalModule = jest.requireActual("../constants");
    return {
        __esModule: true,
        ...originalModule,
        getPricePair: jest.fn().mockReturnValue(originalModule.PRICE_PAIRS_MAINNET),
        getReUSDPricePair: jest.fn().mockReturnValue(originalModule.REUSD_PRICE_MAINNET),
        getPDA: jest.fn().mockReturnValue(originalModule.PDA_ORACLE_MAINNET),
        getRelendInfo: jest.fn().mockReturnValue(originalModule.RELEND_INFO_MAINNET),
    };
});
const RESERVE_RE_BTC_MAINNET = "AtiGYDhwF7WgxCCkoBrBZExGgvkMdZvjF8RbeiFz5JYn";
const RESERVE_RE_VND_MAINNET = "HrViAzmrXZVtvhP6JPSyvcw1EqWNWfUwdVAPfyMwU3rU";
describe("fetchPrices", () => {
    const OLD_ENV = process.env;
    // Common setup for the tests
    let mockMainnetConnection;
    let mockReserveParsed;
    const MOCK_PARSE_PRICE_RETURN_VALUE = {
        version: 1,
        status: { online: {} },
        productAccount: new web3_js_1.PublicKey("4kn3JeaXhBbbwKF3kaYubiWHVCxkJdJap6zsm54CS4YQ"),
        prevPrice: new bn_js_1.BN("43233463396"),
        price: new bn_js_1.BN("43235682627"),
        numPublishers: 1,
        prevTimestamp: new bn_js_1.BN("1705420804"),
        timestamp: new bn_js_1.BN("1705420824"),
        bump: [255],
    };
    const MOCK_PARSE_PRODUCT_RETURN_VALUE = {
        version: 1,
        status: { online: {} },
        assetType: { forex: {} },
        quoteCurrency: "reUSD",
        quoteMint: new web3_js_1.PublicKey("USDbgSB1DPBCEDt15ppgTZZYiQokMHcvkX6JQRfVNJY"),
        baseCurrency: "reBTC",
        baseMint: new web3_js_1.PublicKey("BTCxWeEcT5sYsys7ncnr2bU1Mzbn1aXXqJDv43X22cDy"),
        priceAccount: new web3_js_1.PublicKey("EWMPWwzt5wFWmtsYhEvGxZ8EvEsPp9Fm5rfQBwoXR3Nz"),
        expo: -6,
        maxPrice: new bn_js_1.BN("174876e800", 16),
        minPrice: new bn_js_1.BN("2540be400", 16),
        windowSize: new bn_js_1.BN("14", 16),
        controller: new web3_js_1.PublicKey("B3cWF2ZF5ZJ1bjZfxmKUdpUGR7JDSvJens2bZsmWBnnk"),
        bump: [255],
    };
    beforeEach(async () => {
        // Reset mocks and set default return values
        jest.mocked(custom_oracle_1.parsePriceOracle).mockResolvedValue(MOCK_PARSE_PRICE_RETURN_VALUE);
        jest.mocked(custom_oracle_1.parseProductOracle).mockResolvedValue(MOCK_PARSE_PRODUCT_RETURN_VALUE);
        // Set up mock data
        mockMainnetConnection = new web3_js_1.Connection(constants_1.MAINNET_URL);
        const filters = [{ dataSize: 619 }];
        const rawReserves = await mockMainnetConnection.getProgramAccounts(classes_1.RELEND_PRODUCTION_PROGRAM_ID, {
            commitment: mockMainnetConnection.commitment,
            filters,
            encoding: "base64",
        });
        const parsedReserves = rawReserves
            .map((reserve, index) => (reserve ? (0, state_1.parseReserve)(rawReserves[index].pubkey, reserve.account) : null))
            .filter(Boolean);
        mockReserveParsed = parsedReserves;
    });
    afterAll(() => {
        process.env = OLD_ENV;
    });
    it("should return correct prices", async () => {
        const prices = await (0, prices_1.fetchPrices)(mockReserveParsed, mockMainnetConnection);
        expect(prices).toHaveProperty(RESERVE_RE_BTC_MAINNET);
        expect(prices[RESERVE_RE_BTC_MAINNET]).toEqual({ spotPrice: 43235.682626999995, emaPrice: 43235.682626999995 });
    });
    it("should return default price if price is null/undefined/0 for normal token and fiat token", async () => {
        jest.mocked(custom_oracle_1.parsePriceOracle).mockResolvedValue(null);
        const prices = await (0, prices_1.fetchPrices)(mockReserveParsed, mockMainnetConnection);
        console.log(prices);
        expect(prices).toHaveProperty(RESERVE_RE_BTC_MAINNET);
        expect(prices[RESERVE_RE_BTC_MAINNET]).toEqual({
            spotPrice: prices_1.DEFAULT_PRICES[constants_1.RE_BTC],
            emaPrice: prices_1.DEFAULT_PRICES[constants_1.RE_BTC],
        });
        expect(prices).toHaveProperty(RESERVE_RE_VND_MAINNET);
        expect(prices[RESERVE_RE_VND_MAINNET]).toEqual({
            spotPrice: prices_1.DEFAULT_PRICES[constants_1.RE_VND],
            emaPrice: prices_1.DEFAULT_PRICES[constants_1.RE_VND],
        });
    });
});
