"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unpackAccount = exports.ACCOUNT_SIZE = exports.AccountLayout = void 0;
const buffer_layout_1 = require("@solana/buffer-layout");
const buffer_layout_utils_1 = require("@solana/buffer-layout-utils");
const spl_token_1 = require("@solana/spl-token");
const account_1 = require("./account.js");
const accountType_1 = require("./accountType.js");
const errors_1 = require("./errors.js");
const multisig_1 = require("./multisig.js");
/** Buffer layout for de/serializing a token account */
exports.AccountLayout = (0, buffer_layout_1.struct)([
    (0, buffer_layout_utils_1.publicKey)("mint"),
    (0, buffer_layout_utils_1.publicKey)("owner"),
    (0, buffer_layout_utils_1.u64)("amount"),
    (0, buffer_layout_1.u32)("delegateOption"),
    (0, buffer_layout_utils_1.publicKey)("delegate"),
    (0, buffer_layout_1.u8)("state"),
    (0, buffer_layout_1.u32)("isNativeOption"),
    (0, buffer_layout_utils_1.u64)("isNative"),
    (0, buffer_layout_utils_1.u64)("delegatedAmount"),
    (0, buffer_layout_1.u32)("closeAuthorityOption"),
    (0, buffer_layout_utils_1.publicKey)("closeAuthority"),
]);
exports.ACCOUNT_SIZE = exports.AccountLayout.span;
/**
 * Unpack a token account
 *
 * @param address   Token account
 * @param info      Token account data
 * @param programId SPL Token program account
 *
 * @return Unpacked token account
 */
function unpackAccount(address, info, programId = spl_token_1.TOKEN_PROGRAM_ID) {
    if (!info)
        throw new errors_1.TokenAccountNotFoundError();
    if (!info.owner.equals(programId))
        throw new errors_1.TokenInvalidAccountOwnerError();
    if (info.data.length < exports.ACCOUNT_SIZE)
        throw new errors_1.TokenInvalidAccountSizeError();
    const rawAccount = exports.AccountLayout.decode(info.data.slice(0, exports.ACCOUNT_SIZE));
    let tlvData = Buffer.alloc(0);
    if (info.data.length > exports.ACCOUNT_SIZE) {
        if (info.data.length === multisig_1.MULTISIG_SIZE)
            throw new errors_1.TokenInvalidAccountSizeError();
        if (info.data[exports.ACCOUNT_SIZE] != accountType_1.AccountType.Account)
            throw new errors_1.TokenInvalidAccountError();
        tlvData = info.data.slice(exports.ACCOUNT_SIZE + accountType_1.ACCOUNT_TYPE_SIZE);
    }
    return {
        address,
        mint: rawAccount.mint,
        owner: rawAccount.owner,
        amount: rawAccount.amount,
        delegate: rawAccount.delegateOption ? rawAccount.delegate : null,
        delegatedAmount: rawAccount.delegatedAmount,
        isInitialized: rawAccount.state !== account_1.AccountState.Uninitialized,
        isFrozen: rawAccount.state === account_1.AccountState.Frozen,
        isNative: !!rawAccount.isNativeOption,
        rentExemptReserve: rawAccount.isNativeOption ? rawAccount.isNative : null,
        closeAuthority: rawAccount.closeAuthorityOption ? rawAccount.closeAuthority : null,
        tlvData,
    };
}
exports.unpackAccount = unpackAccount;
